from .type_utils import str2bool

translation = {"type":{str:"STRING", int: "INT", float:"DOUBLE", str2bool:"BOOLEAN", None:"ANY"},
                "column_types": {str:"STRING", None:"ANY", int:"NUMBER", float: "NUMBER"}
                }

