


class Dataset:
    def __init__(self, name=None, description=None , columns_type=None):
        self.name = name
        self.description = description
        self.columns_type = columns_type
        